@@ -132,16 +132,14 @@ class Application(Gtk.Application):
     @log
     def _build_app_menu(self):
         action_entries = [
-            ('about', self._about, None),
-            ("help", self._help, ("app.help", ["F1"]))
+            ('about', self._about),
+            ('help', self._help)
         ]
 
-        for action, callback, accel in action_entries:
+        for action, callback in action_entries:
             simple_action = Gio.SimpleAction.new(action, None)
             simple_action.connect('activate', callback)
             self.add_action(simple_action)
-            if accel is not None:
-                self.set_accels_for_action(*accel)
 
     @log
     def _help(self, action, param):
@@ -160,17 +158,22 @@ class Application(Gtk.Application):
     def do_startup(self):
         Gtk.Application.do_startup(self)
 
+        quit_action = Gio.SimpleAction.new("quit", None)
+        quit_action.connect("activate", self._quit)
+        self.add_action(quit_action)
+        self.set_accels_for_action("app.quit", ["<Ctrl>Q"])
+
         self._build_app_menu()
 
     @log
-    def quit(self, action=None, param=None):
+    def _quit(self, action=None, param=None):
         self._window.destroy()
 
     def do_activate(self):
         if not self._window:
             self._window = Window(self)
             self._window.set_default_icon_name(self.props.application_id)
-            if self.props.application_id == 'org.gnome.MusicDevel':
+            if self.props.application_id == "org.gnome.Music.Devel":
                 self._window.get_style_context().add_class('devel')
             MPRIS(self)
 
