@@ -31,12 +31,15 @@ from gnomemusic.coreartist import CoreArtist
 from gnomemusic.coredisc import CoreDisc
 from gnomemusic.coresong import CoreSong
 from gnomemusic.grilowrappers.grltrackerplaylists import GrlTrackerPlaylists
+from gnomemusic.trackerwrapper import TrackerWrapper
 
 
 class GrlTrackerWrapper(GObject.GObject):
     """Wrapper for the Grilo Tracker source.
     """
 
+    _SPLICE_SIZE = 100
+
     METADATA_KEYS = [
         Grl.METADATA_KEY_ALBUM,
         Grl.METADATA_KEY_ALBUM_ARTIST,
@@ -62,20 +65,21 @@ class GrlTrackerWrapper(GObject.GObject):
     def __repr__(self):
         return "<GrlTrackerWrapper>"
 
-    def __init__(self, source, coremodel, coreselection, grilo):
+    def __init__(
+            self, source, coremodel, application, grilo, tracker_wrapper):
         """Initialize the Tracker wrapper
 
         :param Grl.TrackerSource source: The Tracker source to wrap
         :param CoreModel coremodel: CoreModel instance to use models
         from
-        :param CoreSelection coreselection: CoreSelection instance to
-        use
+        :param Application application: Application instance
         :param CoreGrilo grilo: The CoreGrilo instance
+        :param TrackerWrapper tracker_wrapper: The TrackerWrapper instance
         """
         super().__init__()
 
         self._coremodel = coremodel
-        self._coreselection = coreselection
+        self._coreselection = application.props.coreselection
         self._grilo = grilo
         self._source = source
         self._model = self._coremodel.props.songs
@@ -89,6 +93,7 @@ class GrlTrackerWrapper(GObject.GObject):
         self._artist_search_model = self._coremodel.props.artists_search
         self._batch_changed_media_ids = {}
         self._content_changed_timeout = None
+        self._window = application.props.window
 
         self._song_search_tracker = Gfm.FilterListModel.new(self._model)
         self._song_search_tracker.set_filter_func(lambda a: False)
@@ -103,7 +108,7 @@ class GrlTrackerWrapper(GObject.GObject):
         self._initial_artists_fill(self._source)
 
         self._tracker_playlists = GrlTrackerPlaylists(
-            source, coremodel, coreselection, grilo)
+            source, coremodel, application, grilo, tracker_wrapper)
 
         self._source.notify_change_start()
         self._source.connect("content-changed", self._batch_content_changed)
@@ -113,27 +118,6 @@ class GrlTrackerWrapper(GObject.GObject):
     def source(self):
         return self._source
 
-    @staticmethod
-    def _location_filter():
-        try:
-            music_dir = GLib.get_user_special_dir(
-                GLib.UserDirectory.DIRECTORY_MUSIC)
-            assert music_dir is not None
-        except (TypeError, AssertionError):
-            print("XDG Music dir is not set")
-            return
-
-        music_dir = Tracker.sparql_escape_string(
-            GLib.filename_to_uri(music_dir))
-
-        query = """
-        FILTER (STRSTARTS(nie:url(?song), '%(music_dir)s/'))
-        """.replace('\n', ' ').strip() % {
-            'music_dir': music_dir
-        }
-
-        return query
-
     def _batch_content_changed(self, source, medias, change_type, loc_unknown):
         if medias == []:
             return
@@ -180,6 +164,7 @@ class GrlTrackerWrapper(GObject.GObject):
             ?composer AS ?composer
             ?album_artist AS ?album_artist
             nmm:artistName(?performer) AS ?artist
+            nie:url(?song) AS ?url
             YEAR(MAX(nie:contentCreated(?song))) AS ?creation_date
         WHERE {
             ?album a nmm:MusicAlbum .
@@ -191,7 +176,7 @@ class GrlTrackerWrapper(GObject.GObject):
             %(location_filter)s
         } GROUP BY ?album
         """.replace('\n', ' ').strip() % {
-            'location_filter': self._location_filter()
+            'location_filter': TrackerWrapper.location_filter()
         }
 
         def check_album_cb(source, op_id, media, user_data, error):
@@ -216,7 +201,11 @@ class GrlTrackerWrapper(GObject.GObject):
                 self._album_ids = album_ids
                 return
 
-            album = CoreAlbum(media, self._coremodel)
+            if media.get_id() in self._album_ids.keys():
+                album = self._album_ids[media.get_id()]
+            else:
+                album = CoreAlbum(media, self._coremodel)
+
             album_ids[media.get_id()] = album
 
         options = self._fast_options.copy()
@@ -229,18 +218,24 @@ class GrlTrackerWrapper(GObject.GObject):
 
         query = """
         SELECT
-            rdf:type(?artist_class)
-            tracker:id(?artist_class) AS ?id
-            nmm:artistName(?artist_class) AS ?artist
+            rdf:type(?artist)
+            COALESCE(tracker:id(?album_artist), tracker:id(?artist)) AS ?id
+            ?artist_bind AS ?artist
         WHERE {
-            ?artist_class a nmm:Artist .
             ?song a nmm:MusicPiece;
                     nmm:musicAlbum ?album;
-                    nmm:performer ?artist_class .
+                    nmm:performer ?artist .
+            OPTIONAL {
+                ?album a nmm:MusicAlbum;
+                         nmm:albumArtist ?album_artist .
+            }
+            BIND(COALESCE(nmm:artistName(?album_artist),
+                          nmm:artistName(?artist)) AS ?artist_bind)
             %(location_filter)s
-        } GROUP BY ?artist_class
+        }
+        GROUP BY ?artist_bind
         """.replace('\n', ' ').strip() % {
-            'location_filter': self._location_filter()
+            'location_filter': TrackerWrapper.location_filter()
         }
 
         def check_artist_cb(source, op_id, media, user_data, error):
@@ -265,7 +260,11 @@ class GrlTrackerWrapper(GObject.GObject):
                 self._artist_ids = artist_ids
                 return
 
-            artist = CoreArtist(media, self._coremodel)
+            if media.get_id() in self._artist_ids.keys():
+                artist = self._artist_ids[media.get_id()]
+            else:
+                artist = CoreArtist(media, self._coremodel)
+
             artist_ids[media.get_id()] = artist
 
         options = self._fast_options.copy()
@@ -318,7 +317,7 @@ class GrlTrackerWrapper(GObject.GObject):
             %(location_filter)s
         }
         """.replace('\n', ' ').strip() % {
-            'location_filter': self._location_filter(),
+            'location_filter': TrackerWrapper.location_filter(),
             'media_ids': media_ids
         }
 
@@ -349,18 +348,26 @@ class GrlTrackerWrapper(GObject.GObject):
             options, _update_changed_media)
 
     def _initial_songs_fill(self, source):
+        self._window.notifications_popup.push_loading()
+        songs_added = []
 
         def _add_to_model(source, op_id, media, user_data, error):
             if error:
                 print("ERROR", error)
+                self._window.notifications_popup.pop_loading()
                 return
 
             if not media:
+                self._model.splice(self._model.get_n_items(), 0, songs_added)
+                self._window.notifications_popup.pop_loading()
                 return
 
             song = CoreSong(media, self._coreselection, self._grilo)
-            self._model.append(song)
+            songs_added.append(song)
             self._hash[media.get_id()] = song
+            if len(songs_added) == self._SPLICE_SIZE:
+                self._model.splice(self._model.get_n_items(), 0, songs_added)
+                songs_added.clear()
 
         query = """
         SELECT
@@ -388,25 +395,35 @@ class GrlTrackerWrapper(GObject.GObject):
         }
         ORDER BY ?title
         """.replace('\n', ' ').strip() % {
-            'location_filter': self._location_filter()
+            'location_filter': TrackerWrapper.location_filter()
         }
 
         options = self._fast_options.copy()
         self._source.query(query, self.METADATA_KEYS, options, _add_to_model)
 
     def _initial_albums_fill(self, source):
+        self._window.notifications_popup.push_loading()
+        albums_added = []
 
         def _add_to_albums_model(source, op_id, media, user_data, error):
             if error:
                 print("ERROR", error)
+                self._window.notifications_popup.pop_loading()
                 return
 
             if not media:
+                self._albums_model.splice(
+                    self._albums_model.get_n_items(), 0, albums_added)
+                self._window.notifications_popup.pop_loading()
                 return
 
             album = CoreAlbum(media, self._coremodel)
-            self._albums_model.append(album)
             self._album_ids[media.get_id()] = album
+            albums_added.append(album)
+            if len(albums_added) == self._SPLICE_SIZE:
+                self._albums_model.splice(
+                    self._albums_model.get_n_items(), 0, albums_added)
+                albums_added.clear()
 
         query = """
         SELECT
@@ -416,6 +433,7 @@ class GrlTrackerWrapper(GObject.GObject):
             ?composer AS ?composer
             ?album_artist AS ?album_artist
             nmm:artistName(?performer) AS ?artist
+            nie:url(?song) AS ?url
             YEAR(MAX(nie:contentCreated(?song))) AS ?creation_date
         WHERE
         {
@@ -430,7 +448,7 @@ class GrlTrackerWrapper(GObject.GObject):
         GROUP BY ?album
         ORDER BY ?title ?album_artist ?artist ?creation_date
         """.replace('\n', ' ').strip() % {
-            'location_filter': self._location_filter()
+            'location_filter': TrackerWrapper.location_filter()
         }
 
         options = self._fast_options.copy()
@@ -438,36 +456,51 @@ class GrlTrackerWrapper(GObject.GObject):
         source.query(query, self.METADATA_KEYS, options, _add_to_albums_model)
 
     def _initial_artists_fill(self, source):
+        self._window.notifications_popup.push_loading()
+        artists_added = []
 
         def _add_to_artists_model(source, op_id, media, user_data, error):
             if error:
                 print("ERROR", error)
+                self._window.notifications_popup.pop_loading()
                 return
 
             if not media:
+                self._artists_model.splice(
+                    self._artists_model.get_n_items(), 0, artists_added)
                 self._coremodel.emit("artists-loaded")
+                self._window.notifications_popup.pop_loading()
                 return
 
             artist = CoreArtist(media, self._coremodel)
-            self._artists_model.append(artist)
             self._artist_ids[media.get_id()] = artist
+            artists_added.append(artist)
+            if len(artists_added) == self._SPLICE_SIZE:
+                self._artists_model.splice(
+                    self._artists_model.get_n_items(), 0, artists_added)
+                artists_added.clear()
 
         query = """
         SELECT
             rdf:type(?artist)
-            tracker:id(?artist) AS ?id
-            nmm:artistName(?artist) AS ?artist
+            COALESCE(tracker:id(?album_artist), tracker:id(?artist)) AS ?id
+            ?artist_bind AS ?artist
         WHERE {
-            ?artist a nmm:Artist .
             ?song a nmm:MusicPiece;
                     nmm:musicAlbum ?album;
                     nmm:performer ?artist .
+            OPTIONAL {
+                ?album a nmm:MusicAlbum;
+                         nmm:albumArtist ?album_artist .
+            }
+            BIND(COALESCE(nmm:artistName(?album_artist),
+                          nmm:artistName(?artist)) AS ?artist_bind)
             %(location_filter)s
         }
-        GROUP BY ?artist
-        ORDER BY ?artist
+        GROUP BY ?artist_bind
+        ORDER BY ?artist_bind
         """.replace('\n', ' ').strip() % {
-            'location_filter': self._location_filter()
+            'location_filter': TrackerWrapper.location_filter()
         }
 
         options = self._fast_options.copy()
@@ -479,8 +512,9 @@ class GrlTrackerWrapper(GObject.GObject):
         """Get all albums by an artist
 
         :param Grl.Media media: The media with the artist id
-        :param Dazzle.ListModelFilter model: The model to fill
+        :param Gfm.FilterListModel model: The model to fill
         """
+        self._window.notifications_popup.push_loading()
         artist_id = media.get_id()
 
         query = """
@@ -503,7 +537,7 @@ class GrlTrackerWrapper(GObject.GObject):
         ORDER BY ?date ?album
         """.replace('\n', ' ').strip() % {
             'artist_id': int(artist_id),
-            'location_filter': self._location_filter()
+            'location_filter': TrackerWrapper.location_filter()
         }
 
         albums = []
@@ -511,10 +545,12 @@ class GrlTrackerWrapper(GObject.GObject):
         def query_cb(source, op_id, media, user_data, error):
             if error:
                 print("ERROR", error)
+                self._window.notifications_popup.pop_loading()
                 return
 
             if not media:
                 model.set_filter_func(albums_filter, albums)
+                self._window.notifications_popup.pop_loading()
                 return
 
             albums.append(media)
@@ -536,6 +572,7 @@ class GrlTrackerWrapper(GObject.GObject):
         :param Grl.Media media: The media with the album id
         :param Gfm.SortListModel disc_model: The model to fill
         """
+        self._window.notifications_popup.push_loading()
         album_id = media.get_id()
 
         query = """
@@ -552,15 +589,17 @@ class GrlTrackerWrapper(GObject.GObject):
         ORDER BY ?album_disc_number
         """.replace('\n', ' ').strip() % {
             'album_id': int(album_id),
-            'location_filter': self._location_filter()
+            'location_filter': TrackerWrapper.location_filter()
         }
 
         def _disc_nr_cb(source, op_id, media, user_data, error):
             if error:
                 print("ERROR", error)
+                self._window.notifications_popup.pop_loading()
                 return
 
             if not media:
+                self._window.notifications_popup.pop_loading()
                 return
 
             disc_nr = media.get_album_disc_number()
@@ -611,7 +650,7 @@ class GrlTrackerWrapper(GObject.GObject):
         """.replace('\n', ' ').strip() % {
             'album_id': album_id,
             'disc_nr': disc_nr,
-            'location_filter': self._location_filter()
+            'location_filter': TrackerWrapper.location_filter()
         }
 
         options = self._fast_options.copy()
@@ -628,6 +667,7 @@ class GrlTrackerWrapper(GObject.GObject):
                 GLib.utf8_casefold(text, -1), -1, GLib.NormalizeMode.NFKD))
 
         # Artist search
+        self._window.notifications_popup.push_loading()
 
         query = """
         SELECT DISTINCT
@@ -657,7 +697,7 @@ class GrlTrackerWrapper(GObject.GObject):
         }
         LIMIT 50
         """.replace('\n', ' ').strip() % {
-            'location_filter': self._location_filter(),
+            'location_filter': TrackerWrapper.location_filter(),
             'name': term
         }
 
@@ -669,10 +709,12 @@ class GrlTrackerWrapper(GObject.GObject):
         def artist_search_cb(source, op_id, media, data, error):
             if error:
                 print("ERROR", error)
+                self._window.notifications_popup.pop_loading()
                 return
 
             if not media:
                 self._artist_search_model.set_filter_func(artist_filter)
+                self._window.notifications_popup.pop_loading()
                 return
 
             artist_filter_ids.append(media.get_id())
@@ -682,6 +724,7 @@ class GrlTrackerWrapper(GObject.GObject):
             query, self.METADATA_KEYS, options, artist_search_cb)
 
         # Album search
+        self._window.notifications_popup.push_loading()
 
         query = """
         SELECT DISTINCT
@@ -709,7 +752,7 @@ class GrlTrackerWrapper(GObject.GObject):
         }
         LIMIT 50
         """.replace('\n', ' ').strip() % {
-            'location_filter': self._location_filter(),
+            'location_filter': TrackerWrapper.location_filter(),
             'name': term
         }
 
@@ -721,10 +764,12 @@ class GrlTrackerWrapper(GObject.GObject):
         def albums_search_cb(source, op_id, media, data, error):
             if error:
                 print("ERROR", error)
+                self._window.notifications_popup.pop_loading()
                 return
 
             if not media:
                 self._album_search_model.set_filter_func(album_filter)
+                self._window.notifications_popup.pop_loading()
                 return
 
             album_filter_ids.append(media.get_id())
@@ -734,6 +779,7 @@ class GrlTrackerWrapper(GObject.GObject):
             query, self.METADATA_KEYS, options, albums_search_cb)
 
         # Song search
+        self._window.notifications_popup.push_loading()
 
         query = """
         SELECT DISTINCT
@@ -767,7 +813,7 @@ class GrlTrackerWrapper(GObject.GObject):
         }
         LIMIT 50
         """.replace('\n', ' ').strip() % {
-            'location_filter': self._location_filter(),
+            'location_filter': TrackerWrapper.location_filter(),
             'name': term
         }
 
@@ -779,10 +825,12 @@ class GrlTrackerWrapper(GObject.GObject):
         def songs_search_cb(source, op_id, media, data, error):
             if error:
                 print("ERROR", error)
+                self._window.notifications_popup.pop_loading()
                 return
 
             if not media:
                 self._song_search_tracker.set_filter_func(songs_filter)
+                self._window.notifications_popup.pop_loading()
                 return
 
             filter_ids.append(media.get_id())
@@ -818,6 +866,9 @@ class GrlTrackerWrapper(GObject.GObject):
         SELECT DISTINCT
             rdf:type(?album)
             tracker:id(?album) AS ?id
+            tracker:referenceIdentifier(?release_group_id)
+                AS ?mb_release_group_id
+            tracker:referenceIdentifier(?release_id) AS ?mb_release_id
             tracker:coalesce(nmm:artistName(?album_artist),
                              nmm:artistName(?song_artist)) AS ?artist
             nie:title(?album) AS ?album
@@ -826,6 +877,16 @@ class GrlTrackerWrapper(GObject.GObject):
             ?song a nmm:MusicPiece ;
                     nmm:musicAlbum ?album ;
                     nmm:performer ?song_artist .
+            OPTIONAL {
+                ?album tracker:hasExternalReference ?release_group_id .
+                ?release_group_id tracker:referenceSource
+                    "https://musicbrainz.org/doc/Release_Group" .
+            }
+            OPTIONAL {
+                ?album tracker:hasExternalReference ?release_id .
+                ?release_id tracker:referenceSource
+                    "https://musicbrainz.org/doc/Release" .
+            }
             OPTIONAL { ?album nmm:albumArtist ?album_artist . }
             FILTER (
                 tracker:id(?album) = %(album_id)s
@@ -834,7 +895,7 @@ class GrlTrackerWrapper(GObject.GObject):
         }
         """.replace("\n", " ").strip() % {
                 'album_id': album_id,
-                'location_filter': self._location_filter()
+                'location_filter': TrackerWrapper.location_filter()
         }
 
         return query
@@ -845,6 +906,9 @@ class GrlTrackerWrapper(GObject.GObject):
         SELECT DISTINCT
             rdf:type(?album)
             tracker:id(?album) AS ?id
+            tracker:referenceIdentifier(?release_group_id)
+                AS ?mb_release_group_id
+            tracker:referenceIdentifier(?release_id) AS ?mb_release_id
             tracker:coalesce(nmm:artistName(?album_artist),
                              nmm:artistName(?song_artist)) AS ?artist
             nie:title(?album) AS ?album
@@ -852,6 +916,16 @@ class GrlTrackerWrapper(GObject.GObject):
             ?song a nmm:MusicPiece ;
                     nmm:musicAlbum ?album ;
                     nmm:performer ?song_artist .
+            OPTIONAL {
+                ?album tracker:hasExternalReference ?release_group_id .
+                ?release_group_id tracker:referenceSource
+                    "https://musicbrainz.org/doc/Release_Group" .
+            }
+            OPTIONAL {
+                ?album tracker:hasExternalReference ?release_id .
+                ?release_id tracker:referenceSource
+                    "https://musicbrainz.org/doc/Release" .
+            }
             OPTIONAL { ?album nmm:albumArtist ?album_artist . }
             FILTER (
                 tracker:id(?song) = %(song_id)s
@@ -863,7 +937,7 @@ class GrlTrackerWrapper(GObject.GObject):
             %(location_filter)s
         }
         """.replace("\n", " ").strip() % {
-            'location_filter': self._location_filter(),
+            'location_filter': TrackerWrapper.location_filter(),
             'song_id': song_id
         }
 
