@@ -50,12 +50,6 @@ class RepeatMode(IntEnum):
     SHUFFLE = 3
 
 
-class PlayerField(IntEnum):
-    """Enum for player model fields"""
-    SONG = 0
-    VALIDATION = 1
-
-
 class PlayerPlaylist(GObject.GObject):
     """PlayerPlaylist object
 
@@ -380,6 +374,10 @@ class Player(GObject.GObject):
 
         self._playlist = PlayerPlaylist(self._app)
 
+        self._playlist_model = self._app.props.coremodel.props.playlist_sort
+        self._playlist_model.connect(
+            "items-changed", self._on_playlist_model_items_changed)
+
         self._settings = application.props.settings
         self._settings.connect(
             'changed::repeat', self._on_repeat_setting_changed)
@@ -435,6 +433,11 @@ class Player(GObject.GObject):
         """
         return self.props.state == Playback.PLAYING
 
+    def _on_playlist_model_items_changed(self, model, pos, removed, added):
+        if (removed > 0
+                and model.get_n_items() == 0):
+            self.stop()
+
     @log
     def _on_about_to_finish(self, klass):
         if self.props.has_next:
