@@ -43,7 +43,6 @@ class ArtistAlbumWidget(Gtk.Box):
     selection_mode = GObject.Property(type=bool, default=False)
 
     __gsignals__ = {
-        "ready": (GObject.SignalFlags.RUN_FIRST, None, ()),
         "song-activated": (
             GObject.SignalFlags.RUN_FIRST, None, (SongWidget, )
         ),
@@ -54,7 +53,7 @@ class ArtistAlbumWidget(Gtk.Box):
 
     def __init__(
             self, corealbum, selection_mode_allowed, size_group=None,
-            cover_size_group=None, window=None):
+            cover_size_group=None):
         super().__init__(orientation=Gtk.Orientation.HORIZONTAL)
 
         self._size_group = size_group
@@ -85,12 +84,12 @@ class ArtistAlbumWidget(Gtk.Box):
         if self._cover_size_group:
             self._cover_size_group.add_widget(self._cover_stack)
 
-        self._nb_disc_box_loaded = 0
-        self._model = corealbum.props.model
-        self._model.props.model.connect_after(
+        corealbum.props.model.connect_after(
             "items-changed", self._on_model_items_changed)
         self._disc_list_box.bind_model(
-            self._model, self._create_widget)
+            corealbum.props.model, self._create_widget)
+
+        corealbum.props.model.items_changed(0, 0, 0)
 
     def _create_widget(self, disc):
         disc_box = self._create_disc_box(disc.props.disc_nr, disc.model)
@@ -103,31 +102,17 @@ class ArtistAlbumWidget(Gtk.Box):
         disc_box.props.show_durations = False
         disc_box.props.show_favorites = False
         disc_box.props.show_song_numbers = True
-        disc_box.connect("ready", self._on_discbox_ready)
         disc_box.connect('song-activated', self._song_activated)
 
         return disc_box
 
-    def _on_discbox_ready(self, klass):
-        self._nb_disc_box_loaded += 1
-        if self._nb_disc_box_loaded == self._model.get_n_items():
-            klass.disconnect_by_func(self._on_discbox_ready)
-            self._nb_disc_box_loaded = 0
-            self.emit("ready")
-
     def _on_model_items_changed(self, model, position, removed, added):
         n_items = model.get_n_items()
-        if n_items == 1:
-            row = self._disc_list_box.get_row_at_index(0)
+        for i in range(n_items):
+            row = self._disc_list_box.get_row_at_index(i)
             row.props.selectable = False
             discbox = row.get_child()
-            discbox.props.show_disc_label = False
-        else:
-            for i in range(n_items):
-                row = self._disc_list_box.get_row_at_index(i)
-                row.props.selectable = False
-                discbox = row.get_child()
-                discbox.props.show_disc_label = True
+            discbox.props.show_disc_label = (n_items > 1)
 
     def _song_activated(self, widget, song_widget):
         if self.props.selection_mode:
@@ -144,9 +129,3 @@ class ArtistAlbumWidget(Gtk.Box):
     def select_none(self):
         """Deselect all items"""
         self._disc_list_box.select_none()
-
-    @log
-    def get_selected_songs(self):
-        """Return a list of selected songs."""
-        items = self._disc_list_box.get_selected_items()
-        return items
